﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GhostMovement : MonoBehaviour
{
    Vector3 targetPos;
    public float appearanceSpeed = 0.1f;
    public float ghostDirectionChangeProb = 0.1f;
    public float xMovementRange = 1f;
    public float yMovementRange = 1f;
    public float movementDecreaseByCapture = 0.5f;
    public float speed = 1;
    public float maxXPos = 7;
    public float maxHeight = 9;
    public float initialZpos = 5;
    private IEnumerator initialMoveCoroutine;

    //Pull parameters
    public Transform ghostTrap;
    private bool hitByP1 = false;
    private bool hitByP2 = false;
    Vector3 pullPos;
    float pullStrength = 0;

    //Escape parameters
    float beamP1Time = 0;
    float beamP2Time = 0;
    public float escapeMagnitude = 2f;
    public float captureThreshold = 1f;

    // Use this for initialization
    void Start()
    {
        float initialXpos = Random.Range(-7.5f, 7.5f);
        float initialYpos = Random.Range(7f, maxHeight);
        transform.position = new Vector3(initialXpos, initialYpos, initialZpos);
        targetPos = transform.position;
    }

    // Update is called once per frame
    void Update()
    {
        if (transform.position.z > 0)
        {
            transform.position -= new Vector3(0, 0, appearanceSpeed);
        }

        //Probability of random move changement.
        if (Random.value <= ghostDirectionChangeProb)
            targetPos = transform.position + randomChangeDirection();

        //Update movement
        float step = speed * Time.deltaTime;
        transform.position = Vector3.MoveTowards(transform.position, targetPos, step);
    }

    private Vector3 randomChangeDirection()
    {
        //while next move will move ghost out of bounds, try new move.
        float xDir = ((Random.value * 2) - 1);
        if (transform.position.x + xDir > maxXPos || transform.position.x + xDir < -maxXPos)
            xDir = 0;
        float yDir = ((Random.value * 2) - 1);
        if (transform.position.y + yDir > maxHeight || transform.position.y + yDir < 0.5f * maxHeight)
            yDir = 0;

        //Make movement vector
        return new Vector3(xDir * xMovementRange, yDir * yMovementRange);
    }

    void OnTriggerEnter(Collider col)
    {
        if (col.tag == "BeamP1")
            hitByP1 = true;
        //Debug.Log("Hit by Player1!");
        if (col.tag == "BeamP2")
            hitByP2 = true;

    }

    void OnTriggerStay(Collider col)
    {
        if (hitByP1)
        {
            beamP1Time += Time.deltaTime;
        }
        if (hitByP2)
        {
            beamP2Time += Time.deltaTime;
        }
        float t = beamP1Time + beamP2Time;
        float p = Random.value * (Mathf.Sin(t) + 1); //function between 0 and 2
        if (p > captureThreshold)
            EscapeFromBeam();
    }

    void OnTriggerExit(Collider col)
    {
        if (col.tag == "BeamP1")
        {
            hitByP1 = false;
            beamP1Time = 0;
        }

        if (col.tag == "BeamP2")
        {
            hitByP2 = false;
            beamP2Time = 0;
        }
            pullStrength = 0;
    }

    void EscapeFromBeam()
    {
        float p = Random.value;
        float y = escapeMagnitude / (Mathf.Sqrt((p * p) + 1));
        float x = p * y;
        Vector3 movement = new Vector3(x, y, 0);
        transform.position += movement;
    }

    void MoveTowards(object[] objects)
    {
        pullPos = (Vector3)objects[0];
        pullStrength = (float)objects[1];

        if(hitByP1 && hitByP2)
            transform.position = Vector3.MoveTowards(transform.position, ghostTrap.position, pullStrength * 2);
            //transform.position = Vector3.MoveTowards(transform.position, new Vector3(transform.position.x, 0, 0), pullStrength * 2);
        else
            transform.position = Vector3.MoveTowards(transform.position, pullPos, pullStrength);
    }

    public bool isHitByP1()
    {
        return hitByP1;
    }

    public bool isHitByP2()
    {
        return hitByP2;
    }
}
