﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(ParticleSystem))]
public class ParticleController : MonoBehaviour {

    ParticleSystem m_System;
    ParticleSystem.Particle[] m_Particles;
    public float m_Drift = 0.01f;

    float timer = 0;
    float mPhase = 0;

    public float Amplitude = 10.0f;
    public float AmplitudeZ = 5.0f;
    public float XIncrement = 1.0f;
    public float dx = 1.0f;
    public float PhaseIncrement = 0.1f;
    public float Width = 1.0f;

    private void LateUpdate()
    {
        InitializeIfNeeded();

        // GetParticles is allocation free because we reuse the m_Particles buffer between updates
        int numParticlesAlive = m_System.GetParticles(m_Particles);

        float t = 0;
        float fy = 0;
        float fz = 0;
        float dl = 0.01f;

        timer += Time.deltaTime;

        mPhase += PhaseIncrement * Time.deltaTime * Mathf.PI;

        // Change only the particles that are alive
        for (int i = 0; i < numParticlesAlive; i++)
        {
            fy = Amplitude * Mathf.Sin(t + mPhase);
            fz = AmplitudeZ * Mathf.Sin(t + mPhase);
            m_Particles[i].position = transform.position + new Vector3(dx * i, fy, fz);
            t += XIncrement * 0.001f * Mathf.PI;
        }

        // Apply the particle changes to the particle system
        m_System.SetParticles(m_Particles, numParticlesAlive);
    }

    void InitializeIfNeeded()
    {
        if (m_System == null)
            m_System = GetComponent<ParticleSystem>();

        if (m_Particles == null || m_Particles.Length < m_System.maxParticles)
            m_Particles = new ParticleSystem.Particle[m_System.maxParticles];
    }
}
