﻿using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using UnityEngine;
using UnityEngine.Networking;
using System.Text.RegularExpressions;

public class Server : MonoBehaviour
{

    public float playerAngle_1 = 0.0f;
    public float playerAngle_2 = 0.0f;

    public float playerMic_1 = 0.0f;
    public float playerMic_2 = 0.0f;

    public bool playerFire_1 = false;
    public bool playerFire_2 = false;


    int mServerSocket = -1;
    int mClientSocket = -1;
    int mClientConnection = -1;
    int mMaxConnections = 10;
    byte mChannelUnreliable;
    byte mChannelReliable;
    bool mHostClientInitialized = false;
    bool mClientConnected = false;

    public void Start()
    {
        // Build global config
        GlobalConfig gc = new GlobalConfig();
        gc.ReactorModel = ReactorModel.FixRateReactor;
        gc.ThreadAwakeTimeout = 10;

        // Build channel config
        ConnectionConfig cc = new ConnectionConfig();
        mChannelReliable = cc.AddChannel(QosType.ReliableSequenced);
        mChannelUnreliable = cc.AddChannel(QosType.UnreliableSequenced);

        // Create host topology from config
        HostTopology ht = new HostTopology(cc, mMaxConnections);

        // We have all of our other stuff figured out, so init
        NetworkTransport.Init(gc);

        // Open sockets for server and client
        mServerSocket = NetworkTransport.AddHost(ht, 8888);

        // Check to make sure our socket formations were successful
        if (mServerSocket < 0)
        {
            Debug.Log("Server socket creation failed!");
        }
        else
        {
            Debug.Log("Server socket creation successful!");
        }
    }

    public void Update()
    {
        int recHostId;
        int connectionId;
        int channelId;
        int dataSize;
        byte[] buffer = new byte[1024];
        byte error;

        NetworkEventType networkEvent = NetworkEventType.DataEvent;

        // Poll both server/client events
        do
        {
            networkEvent = NetworkTransport.Receive(out recHostId, out connectionId, out channelId, buffer, 1024, out dataSize, out error);

            int realConnectionId = (connectionId-1)%2 +1;

            switch (networkEvent)
            {
                case NetworkEventType.Nothing:
                    break;
                case NetworkEventType.ConnectEvent:
                    // Server received disconnect event
                    if (recHostId == mServerSocket)
                    {
                        Debug.Log("Server: Player " + realConnectionId.ToString() + " connected!");
                    }

                    break;

                case NetworkEventType.DataEvent:
                    // Server received data
                    if (recHostId == mServerSocket)
                    {

                        // Let's decode data
                        Stream stream = new MemoryStream(buffer);
                        BinaryFormatter f = new BinaryFormatter();
                        string msg = f.Deserialize(stream).ToString();

                        Debug.Log("Server: Received Data from " + realConnectionId.ToString() + "! Message: " + msg);

                        ParseMsg(msg, realConnectionId);
                    }
                    if (recHostId == mClientSocket)
                    {
                        Debug.Log("Client: Received Data from " + realConnectionId.ToString() + "!");
                    }
                    break;

                case NetworkEventType.DisconnectEvent:
                    // Client received disconnect event
                    if (recHostId == mClientSocket && realConnectionId == mClientConnection)
                    {
                        Debug.Log("Client: Disconnected from server!");

                        // Flag to let client know it can no longer send data
                        mClientConnected = false;
                    }

                    // Server received disconnect event
                    if (recHostId == mServerSocket)
                    {
                        Debug.Log("Server: Received disconnect from " + realConnectionId.ToString());
                    }
                    break;
            }

        } while (networkEvent != NetworkEventType.Nothing);
    }

    void ParseMsg(string msg, int connectionId)
    {
        //msg = "Gyro: (45.5, 6.8, 0.0) # Mic: 0.0008 # Fire: False";

        Regex regex = new Regex(@"[0-9]+.[0-9]+E?-?[0-9]*");
        MatchCollection matches = regex.Matches(msg);

        Regex regex_2 = new Regex(@"True");

        Match match = regex_2.Match(msg);


        if (connectionId == 1)
        {
            playerAngle_1 = FixAngle(float.Parse(matches[0].Value));

            if (matches.Count == 4)
            {
                playerMic_1 = float.Parse(matches[3].Value);
            }
            else
            {
                playerMic_1 = 1f;
            }


            playerFire_1 = match.Success;

        }
        else if (connectionId == 2)
        {
            playerAngle_2 = FixAngle(float.Parse(matches[0].Value));

            if (matches.Count == 4)
            {
                playerMic_2 = float.Parse(matches[3].Value);
            }
            else
            {
                playerMic_2 = 1f;
            }

            playerFire_2 = match.Success;
        }

    }

    float FixAngle(float angle)
    {
        if (angle >= 180.0f && angle <= 360.0f)
        {
            return 0.0f;
        }
        else if (angle >= 90f && angle < 180f)
        {
            return 90.0f;
        }

        return angle;
    }
}
